#include "../defs.h"
#include "../errors.h"
#include "../Lock/lock.h"
#include "../mountrec.h"
#include "../DevSupport/readwrite.h"
#include "partition.h"

#include <stdlib.h>

#define NODEBUG

#ifdef DEBUG
#include <stdio.h>
#endif


int getpartinfo(MOUNT_RECORD *mr, int partnum)
{
  BYTE *sectbuf, *tmp;
  WORD temp;
  LONGWORD templong;

/* Guess at the sector size */

  mr->sr.sectsize=BYTESPERSECT;

  if ((sectbuf=(BYTE *)malloc(BYTESPERSECT))==NULL)
    return PARTOUTOFMEM;
  
  if ((readabsolutesector(mr, 0, sectbuf))) {
    free(sectbuf);
    return PARTCANTREADSECTZERO;
  }


/* This is the driver descriptor map */

  temp=reorderword(&(sectbuf[0x0]));

#ifdef DEBUG
  printf("ID word is %X\n", temp);
#endif

/* If it's not a valid driver descriptor map, assume it an HFS floppy */

  if (temp!=0x4552) {

/* Assume it's a 1.44 MB floppy and let readvolinfo deal with it */

    mr->sr.sectoffsettopartition=0;
    free(sectbuf);
    return 0;

  }

/* OK, so it's a partitioned SCSI device */

/* Get the true sector size */

  temp=reorderword(&(sectbuf[0x2]));

#ifdef DEBUG
  printf("Bytes Per Sector is %d\n", temp);
#endif

  mr->sr.sectsize=temp;

#ifdef DEBUG
  printf("%ul blocks on device\n", reorderlongword(&(sectbuf[4])));
  printf("Continuing to Partition Map Partition..\n");
#endif

  if (temp>BYTESPERSECT)
    if ((tmp=(char *) realloc(sectbuf, temp))==NULL) {
      free(sectbuf);
      return PARTOUTOFMEM;
    } else {
      sectbuf=tmp;
    }

  if (readabsolutesector(mr, partnum, sectbuf)) {
    free(sectbuf);
    return PARTCANTREADPARTMAPSECT;
  }

  temp=reorderword(&(sectbuf[0]));

/* See if it is a valid partition map entry */

  if (temp!=0x504d) {
    free(sectbuf);
    return BADPARTITION;
  }

  templong=reorderlongword(&(sectbuf[8]));

#ifdef DEBUG
  printf("Requested partition starts at sector %lu and has %lu sectors\n",
	 templong, reorderlongword(&(sectbuf[0xc])));
#endif

  mr->sr.sectoffsettopartition=templong;

  sectbuf[0x50]=0;

#ifdef DEBUG
  printf("Partition Type Name is '%s'\n", &(sectbuf[0x30]));
#endif

  if (strcmp(&(sectbuf[0x30]),MACHFSPART)) {
    free(sectbuf);
    return NOTMACHFSPART;
  }


#ifdef DEBUG
  sectbuf[0x30]=0;
  printf("Partition Name is '%s'\n", &(sectbuf[0x10]));
  puts("getpartinfo completed.");
#endif

  free(sectbuf);

  return 0;

}
  
  
  
  







