int maccreatefile(MOUNT_RECORD *mr,
		  char *absolutepathname,
		  int mode,
		  MFILE *mfile);
/* Given valid mount record, absolutepathname, and catalog record, 
   file creation will be attempted.  Nonzero return indicates
   success and *cr will be that files catalog record.
*/

int maccreatefolder(MOUNT_RECORD *mr,
		    char *absolutepathname);
/* See maccreatefile 
*/



int macmove(MOUNT_RECORD *mr,
	    char *absolutepathname,
	    char *newabsolutepathname);
/* Moves the item named by absolutepathname to the location and name
   specified by newabsolutepathname
*/




int macdeletefile(MOUNT_RECORD *mr,
		  char *absolutepathname);
/* Deletes file at absolutepathname.  Returns nonzero on failure.
*/

int macdeletefolder(MOUNT_RECORD *mr,
		    char *absolutename);
/* Deletes folder fn at absolutepathname.  Returns nonzero on failure.
*/




#define READ        1
#define WRITE       2
#define APPEND      4
#define CREATE      8
#define TRUNCATE    16
#define SHARED      32


#define isreadable(mfileptr) ((mfileptr)->mode & 1)
#define iswriteable(mfileptr) ((mfileptr)->mode & 2)
#define isappend(mfileptr)  ((mfileptr)->mode & 4)
#define iscreate(mfileptr)  ((mfileptr)->mode & 8)
#define istruncate(mfileptr)  ((mfileptr)->mode & 16)
#define isshared(mfileptr)  ((mfileptr)->mode & 32)

int macopen(MOUNT_RECORD *mr, 
	    char *absolutepathname,
	    int mode,
	    MFILE *mfile);
/* Given a valid mount record and a pathname starting at :, macopen will 
   fill in the catalog record appropriately and return 0.  For relative
   searches, the cr->dirDirID field should be filled with the directory
   ID of the current directory. '..' and '.' are not supported.
   A nonzero return means
   that no exact match could be found for some reason and cr is the
   closest.  *pid is the parent of what was just opened.  name is the
   last path element
*/


int macclose(MFILE *mfile);
/* Closes files associated with Catalog Record on volume referenced by mr
   Nonzero return on failure
*/


int macdirentries(MOUNT_RECORD *mr,
		  LONGWORD dirid,
		  CATRECANDNAME **child);
/* Given a directory ID, return with an array of structures containing
   names and catalog records of its children.  The integer return is
   negative for errors, and, when postive, reprsents the number of
   children found.
*/

int macsetfinderinfo(MFILE *mfile, 
		     BYTE *UsrWds, 
		     BYTE *FndrInfo);
/* Sets the finder info fields (each is 16 bytes long) of the file MFILE
   represents to the passed in values.  If either field is NULL, it is not
   copied in.  You must macclose the file before the information is 
   flushed to disk.
*/

int macgetfinderinfo(MFILE *mfile, 
		     BYTE *UsrWds, 
		     BYTE *FndrInfo);
/* Gets the finder info fields (each is 16 bytes long) of the file MFILE
   and copies them into the supplied fields.  If either field is NULL, 
   it is not copied out.  
*/


int mdirentries(MOUNT_RECORD *mr,
		LONGWORD dirid,
		CATRECANDNAME **child);



int mcreatefile(MOUNT_RECORD *mr,
		CATALOG_RECORD *parentcr,
		char *name,
		CATALOG_RECORD *cr);

int mcreatefolder(MOUNT_RECORD *mr,
		  CATALOG_RECORD *parentcr,
		  char *name,
		  CATALOG_RECORD *cr);


int mmove(MOUNT_RECORD *mr,
	  CATALOG_RECORD *oldparentcr,
	  char *name,
	  CATALOG_RECORD *cr,
	  CATALOG_RECORD *newparentcr,
	  char *newname);

int mmovefile(MOUNT_RECORD *mr,
	      CATALOG_RECORD *oldparentcr,
	      char *name,
	      CATALOG_RECORD *cr,
	      CATALOG_RECORD *newparentcr,
	      char *newname);

int mmovefolder(MOUNT_RECORD *mr,
		CATALOG_RECORD *oldparentcr,
		char *name,
		CATALOG_RECORD *cr,
		CATALOG_RECORD *newparentcr,
		char *newname);



int mdeletefile(MOUNT_RECORD *mr,
	        CATALOG_RECORD *parentcr,
		char *name,
		CATALOG_RECORD *cr);

int mdeletefolder(MOUNT_RECORD *mr,
		  CATALOG_RECORD *parentcr,
		  char *name,
		  CATALOG_RECORD *cr);

int mopen(MOUNT_RECORD *mr,
	  LONGWORD pid,
	  char *name,
	  CATALOG_RECORD *cr);

int mclose(MOUNT_RECORD *mr,
	   LONGWORD pid,
	   char *name,
	   CATALOG_RECORD *cr);
	  

#define IDOFDIRCATREC(crptr) ((crptr)->U.cdrDirRec.dirDirID)


int mopenroot(MOUNT_RECORD *mr,
	      CATALOG_RECORD *cr,
	      char *name);
/* If name==NULL, then no name of root catalog record is returned.  Otherwise,
   the name is written there
*/


#define ROOTPID 1
#define ROOTID  2


#define LOWESTFN -1

#define BIGESTFN "\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\x00"

#define SMALLESTFN "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\0x00"




