#include "../defs.h"
#include "../errors.h"
#include "../Lock/lock.h"
#include "../mountrec.h"
#include "../btree/btree.h"
#include "../Volume/volbmap.h"
#include "../LibFuncs/macreadwrite_m.h"
#include "../Times/mac_time.h"
#include "catalog_m.h"
#include "catalog_mac.h"
#include "catalog_mac_local.h"
#include "../LibFuncs/catrectable.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

#define NODEBUG

int revertifopencatrec(CATALOG_RECORD *cr)
{
  CATALOG_RECORD *ocr;

  if ((ocr=findidoncatrectable(cr))) {
    memcpy(cr,ocr,sizeof(CATALOG_RECORD));
  }
	
  return 0;

}

int maccreatefile(MOUNT_RECORD *mr,
		  char *absolutepathname,
		  int mode,
		  MFILE *mfile)
{
  int rc;
  CATALOG_RECORD dircr;
  LONGWORD dirpid;
  char parname[MAXFNLENGTH];

  if ((mfile->cr=allocopencatrec())==NULL)
    return TOOMANYOPENFILES;
  
  mfile->mr=mr; mfile->mode=mode;
  

  if ((rc=macresolvepnwithnonexistinglastelement(mr,
						 absolutepathname,
						 &dirpid,
						 parname,
						 &dircr,
						 mfile->name))) {
    deallocopencatrec(mfile->cr);
    return rc;
  }

  if (dircr.cdrType!=CDR_DIR) {
    deallocopencatrec(mfile->cr);
    return CANTCREATEINNONDIR;
  }

  mfile->pid=dircr.U.cdrDirRec.dirDirID;

  lockcatrec(mfile->cr);
  
  if ((rc=mcreatefile(mr, &dircr, mfile->name, mfile->cr))) {
    deallocopencatrec(mfile->cr);
    return rc;
  }

  mfile->cr->refcount=1;
  mfile->cr->status=0;
  
  unlockcatrec(mfile->cr);
  

  if (PutCatalogRecord(mr,
		       dirpid,
		       parname,
		       &dircr))
    return CANTUPDATEDIR;

  if (isappend(mfile))
    mfile->offset=mfile->cr->U.cdrFilRec.filLgLen;
  else
    mfile->offset=0;


  mfile->valid=VALIDVAL;

  return 0;
}




int macsetfinderinfo(MFILE *mfile, BYTE *UsrWds, BYTE *FndrInfo)
{
  if (mfile->valid!=VALIDVAL) 
    return INVALIDMFILE;

  if (mfile->cr->cdrType!=CDR_FILE)
    return CANTSETNONFILEFINDERINFO;

  if (UsrWds!=NULL)
    memcpy(mfile->cr->U.cdrFilRec.filUsrWds, UsrWds, 16);

  if (FndrInfo!=NULL)
    memcpy(mfile->cr->U.cdrFilRec.filFndrInfo, FndrInfo, 16);

  return 0;

}


int macgetfinderinfo(MFILE *mfile, BYTE *UsrWds, BYTE *FndrInfo)
{
  if (mfile->valid!=VALIDVAL) 
    return INVALIDMFILE;

  if (mfile->cr->cdrType!=CDR_FILE)
    return CANTGETNONFILEFINDERINFO;

  if (UsrWds!=NULL)
    memcpy(UsrWds, mfile->cr->U.cdrFilRec.filUsrWds, 16);

  if (FndrInfo!=NULL)
    memcpy(FndrInfo, mfile->cr->U.cdrFilRec.filFndrInfo, 16);

  return 0;

}




#define setcreator(finderinfoptr,creatorptr)                          \
         memcpy(&((finderinfoptr)[4]),creatorptr, 4)           

#define settype(finderinfoptr, creatorptr)    \
         memcpy(finderinfoptr, creatorptr, 4)
    
  

int maccreatefolder(MOUNT_RECORD *mr,
		    char *absolutepathname)
{
  LONGWORD parpid;
  char foldername[MAXFNLENGTH];
  CATALOG_RECORD cr, parcr;
  int rc;
  char parname[MAXFNLENGTH];

  if ((rc=macresolvepnwithnonexistinglastelement(mr,
						 absolutepathname,
						 &parpid,
						 parname,
						 &parcr,
						 foldername)))
    return rc;

  if (parcr.cdrType!=CDR_DIR)
    return CANTCREATEINNONDIR;

  if ((rc=mcreatefolder(mr,&parcr,foldername, &cr)))
    return rc;

  if (PutCatalogRecord(mr,
		       parpid,
		       parname,
		       &parcr))
    return CANTUPDATEDIR;
  
  return 0;

  
}


int macmove(MOUNT_RECORD *mr,
	    char *absolutepathname,
	    char *newabsolutepathname)
{
  int rc;
  CATALOG_RECORD oldparcr, newparcr, cr, *ocr;
  LONGWORD oldparpid, newparpid,oldpid;
  char name[MAXFNLENGTH], newname[MAXFNLENGTH], oldparname[MAXFNLENGTH];
  char newparname[MAXFNLENGTH];

/* Get old parent catalog record */

  if ((rc=macresolvepnwithnonexistinglastelement(mr,
						 absolutepathname,
						 &oldparpid,
						 oldparname,
						 &oldparcr,
						 name)))
    return rc;

/* Get old catalog record */

  if ((rc=macresolvepathname(mr,
			     absolutepathname,
			     &cr,
			     &oldpid,
			     name)))
    return rc;


  ocr=findidoncatrectable(&cr);
  
  if (ocr==NULL) ocr=&cr;

  lockcatrec(ocr);

/* Get new parent catalog record */

  if ((rc=macresolvepnwithnonexistinglastelement(mr,
						 newabsolutepathname,
						 &newparpid,
						 newparname,
						 &newparcr,
						 newname))) {
    unlockcatrec(ocr);
    return rc;
  }

  
  rc=mmove(mr, &oldparcr, name, ocr, &newparcr, newname);

  if (!rc) {
    setstatusofmatchingopencatrecs(ocr,MOVED);
  }

  unlockcatrec(ocr);

  rc=PutCatalogRecord(mr,
		      oldparpid,
		      oldparname,
		      &oldparcr);

  rc+=PutCatalogRecord(mr,
		       newparpid,
		       newparname,
		       &newparcr);

  if (rc) 
    return CANTUPDATEDIR;

  return 0;

}



int macdeletefile(MOUNT_RECORD *mr,
		  char *absolutepathname)
{
  int rc;
  CATALOG_RECORD cr, parcr, *ocr;
  LONGWORD parpid, pid;
  char fn[MAXFNLENGTH], parname[MAXFNLENGTH];


  if ((rc=macresolvepnwithnonexistinglastelement(mr,
						 absolutepathname,
						 &parpid,
						 parname,
						 &parcr,
						 fn)))
    return rc;


  if ((rc=macresolvepathname(mr,
			     absolutepathname,
			     &cr,
			     &pid,
			     fn)))
    return rc;
  
  ocr=findidoncatrectable(&cr);

  if (ocr==NULL)
    ocr=&cr;

  lockcatrec(ocr);

  rc=mdeletefile(mr, &parcr, fn, ocr);

  if (!rc)
    setstatusofmatchingopencatrecs(ocr, DELETED);

  unlockcatrec(ocr);

  if (PutCatalogRecord(mr,
		       parpid,
		       parname,
		       &parcr))
    return CANTUPDATEDIR;

  return rc;

}


int macdeletefolder(MOUNT_RECORD *mr,
		    char *absolutepathname)
{
  int rc;
  CATALOG_RECORD fcr, parcr;
  LONGWORD pid, parpid;
  char name[MAXFNLENGTH], parname[MAXFNLENGTH];


  if ((rc=macresolvepnwithnonexistinglastelement(mr,
						 absolutepathname,
						 &parpid,
						 parname,
						 &parcr,
						 name)))
    return rc;

  if ((rc=macresolvepathname(mr,
			     absolutepathname,
			     &fcr,
			     &pid,
			     name)))
    return rc;


  rc=mdeletefolder(mr, &parcr, name, &fcr);

  if (!rc) {
    if (PutCatalogRecord(mr,
			 parpid,
			 parname,
			 &parcr))
      return CANTUPDATEDIR;
  }

  return rc;

}


int macresolvepathname(MOUNT_RECORD *mr, 
		       char *pathname,
		       CATALOG_RECORD *cr,
		       LONGWORD *pid,
		       char *name)
{
  char *localpathcopy;
  char *pathelement;
  char tokensep[2];
  LONGWORD curpid, lastpid;
  int rc;
  LONGWORD foundpid;
  char fnbuf[MAXFNLENGTH];

  if ((localpathcopy=(char *) malloc(strlen(pathname)+1))==NULL) 
    return PCROUTOFMEMORY;

  strcpy(localpathcopy, pathname);
  
  tokensep[0]=MACDIRSEP; tokensep[1]=0;

  if (*pathname==MACDIRSEP) {

/* ":" should be legal and open the root directory */

    if (strlen(pathname)==1) {
      free(localpathcopy);
      return mopenroot(mr, cr, name);
    } else {
      curpid=ROOTID;cr->cdrType=CDR_DIR;
      pathelement=strtok(localpathcopy+1,tokensep);
    }
  } else {
    pathelement=strtok(localpathcopy,tokensep);
    switch (cr->cdrType) {
    case CDR_DIR:
      curpid=cr->U.cdrDirRec.dirDirID;
      break;
    case CDR_FILE,CDR_DTHREAD,CDR_FTHREAD:
    default:
      free(localpathcopy);
      return PCRCWDNOTDIR;
    }
  }

/* OK, if we got this far, we have a valid cr and can continue normally */

    do {
      lastpid=curpid;
      switch (cr->cdrType) {
      case CDR_FILE: 
	free(localpathcopy);
	return PCRGOTFILENOTDIR;
	break;
      case CDR_DIR:
	if (((strlen(pathelement)<1)||strlen(pathelement)>MAXFNLENGTH)
	    && (curpid!=ROOTPID)) {
	  free(localpathcopy);
	  return PCRBADPATHELEMENT;
	}
	if ((rc=GetCatalogRecord(mr,curpid,pathelement,cr,&foundpid,fnbuf))) {
	  free(localpathcopy);
	  return rc;
	}
	if (((foundpid!=curpid)||(strcasecmp(pathelement,fnbuf)))
	    && (curpid!=ROOTPID)) {
	  free(localpathcopy);
	  return PCRNOSUCHPATHELEMENT;
	}
	break;
      default:
	free(localpathcopy);
	return PCRUNEXPECTEDTYPE;
      }
      if (cr->cdrType==CDR_DIR) {
	curpid=cr->U.cdrDirRec.dirDirID;
      }
    } while ((pathelement=strtok(NULL,tokensep))!=NULL);

/* So now cr should correspond to the last element of the pathname */

  *pid=lastpid;
  strcpy(name,fnbuf);
  free(localpathcopy);

#ifdef DEBUG
  printf("resolved '%s' into pid=%lu, name='%s'\n",
	 pathname, *pid, name);
#endif

  return 0;
}




int macresolvepnwithnonexistinglastelement(MOUNT_RECORD *mr,
					   char *absolutepathname,
					   LONGWORD *parpid,
					   char *parname,
					   CATALOG_RECORD *parcr,
					   char *name)
{
  int pos, rc;

  for (pos=strlen(absolutepathname)-1;pos>=0;pos--) 
    if (absolutepathname[pos]==MACDIRSEP)
      break;

  if (pos==0) {
    if ((rc=mopenroot(mr, parcr, parname)))
	return rc;
    if ((strlen(absolutepathname)>1)&&
	(strlen(absolutepathname)<(MAXFNLENGTH+1))) {
      strcpy(name,&(absolutepathname[1]));
      *parpid=ROOTPID;
      return 0;
    } else {
      *name=0;
      return BADFILEORFOLDERNAME;
    }
  }

  absolutepathname[pos]=0;

  rc=macresolvepathname(mr,
			absolutepathname,
			parcr,
			parpid,
			parname);

  absolutepathname[pos]=MACDIRSEP;

  if (rc)
    return rc;

  strcpy(name, &(absolutepathname[pos+1]));

#ifdef DEBUG
  printf("Resolved '%s' into parpid=%lu, pid=%lu, name='%s'\n",
	 absolutepathname, *parpid, parcr->U.cdrDirRec.dirDirID, name);
#endif

  return 0;

}
    

int macdirentries(MOUNT_RECORD *mr,
		  LONGWORD dirid,
		  CATRECANDNAME **child)
{
  CATRECANDNAME *crn, *tmp;
  CATALOG_RECORD *ocr;
  char queryfn[MAXFNLENGTH];
  int fi, rc;
  LONGWORD foundpid;
  int hitlowestfile;




  if ((crn=(CATRECANDNAME *) 
       malloc(sizeof(CATRECANDNAME)*CRNALLOCUNITS))==NULL) 
    return MACDIROUTOFMEM;

  strcpy(queryfn, BIGESTFN);

  fi=0;
  hitlowestfile=0;

  do {

    if ( !(fi%CRNALLOCUNITS) && (fi!=0) ) {
      if ((tmp=(CATRECANDNAME *)
	   realloc(crn,(fi+CRNALLOCUNITS)*sizeof(CATRECANDNAME)))==NULL) {
	free(crn);
	return MACDIROUTOFMEM;
      } else {
	crn=tmp;
      }
    }

    rc=GetCatalogRecord(mr,
			dirid,
			queryfn,
			&(crn[fi].cr),
			&foundpid,
			(char *) &(crn[fi].name));
    
    if (!rc || (rc==BT_FOUNDLESS && foundpid==dirid)) {

      if (crn[fi].cr.cdrType==CDR_DTHREAD) {
	if (crn[fi].cr.U.cdrThdRec.thdParID==dirid) {
	  break;
	}
      }

      memcpy(queryfn,
	     &(crn[fi].name),
	     MAXFNLENGTH);

      if (reducefn(queryfn)==LOWESTFN) {
	break;
      } 

      ocr=findidoncatrectable(&(crn[fi].cr));

      if (ocr!=NULL) {
	lockcatrec(ocr);
	memcpy(&(crn[fi].cr),ocr,sizeof(CATALOG_RECORD));
	unlockcatrec(ocr);
      }

      ++fi;

    } else {
      free(crn);
      return rc;
    }

  } while ((foundpid==dirid));

  *child=crn;

  return fi;

}

    
int macopen(MOUNT_RECORD *mr,
	    char *absolutepathname,
	    int mode,
	    MFILE *mfile)
{
  int rc;
  CATALOG_RECORD tempcr, *ocr, parcr;
  LONGWORD parpid;
  char parname[MAXFNLENGTH];

  

  mfile->mr=mr; mfile->mode=mode;

  if (*absolutepathname!=MACDIRSEP)
    return BADPATH;

  if ((rc=macresolvepnwithnonexistinglastelement(mr, 
						 absolutepathname, 
						 &parpid,
						 parname,
						 &parcr,
						 mfile->name))) {
    return rc;
  }

  if (parcr.cdrType!=CDR_DIR)
    return BADPATH;

  mfile->pid=parcr.U.cdrDirRec.dirDirID;

  if ((rc=mopen(mr, mfile->pid, mfile->name, &tempcr))) {
    if ((rc==BT_NOTFOUND || rc==BT_FOUNDLESS) && (mode&CREATE)) {
      if ((rc=mcreatefile(mr, &parcr, mfile->name, &tempcr)))
	return rc;
    } else
      return rc;
  }
  
  if (tempcr.cdrType!=CDR_FILE) {
    return CANONLYOPENFILES;
  }


  ocr=findidoncatrectable(&tempcr);

  if (ocr!=NULL) {
    lockcatrec(ocr);
    ++(ocr->refcount);
    mfile->cr=ocr;
    unlockcatrec(ocr);
  } else {
    if ((mfile->cr=allocopencatrec())==NULL) 
      return TOOMANYOPENFILES;
    lockcatrec(mfile->cr);
    memcpy(mfile->cr,&tempcr,sizeof(CATALOG_RECORD));
    mfile->cr->refcount=1;
    mfile->cr->status=0;
    unlockcatrec(mfile->cr);
  }

  if (istruncate(mfile)) {
    if ((rc=mtruncatefile(mr, mfile->cr, 0)))
      return rc;
  }

  if (isappend(mfile)) 
    mfile->offset=mfile->cr->U.cdrFilRec.filLgLen;
  else
    mfile->offset=0;

  mfile->valid=VALIDVAL;

  return 0;

}


int macclose(MFILE *mfile)
{
  int rc;

  if (mfile->valid!=VALIDVAL)
    return FILEISNOTOPEN;

  lockcatrec(mfile->cr);
  --(mfile->cr->refcount);
  if (mfile->cr->refcount==0) {
    if (mfile->cr->status==DELETED || mfile->cr->status==MOVED) {
      unlockcatrec(mfile->cr);
      deallocopencatrec(mfile->cr);
      mfile->valid=0;
      return FILEISGONE;
    } else {
      rc=mclose(mfile->mr, mfile->pid, mfile->name, mfile->cr);
      unlockcatrec(mfile->cr);
      deallocopencatrec(mfile->cr);
      mfile->valid=0;
      return 0;
    }
  }
  unlockcatrec(mfile->cr);

  mfile->valid=0;
  return 0;
}

		     









