#include "../defs.h"
#include "../errors.h"
#include "../Lock/lock.h"
#include "../mountrec.h"
#include "../btree/btree.h"
#include "../LibFuncs/catalog.h"
#include "../DevSupport/readwrite.h"
#include "../Volume/volbmap.h"
#include "../Times/mac_time.h"
#include "macreadwrite_mac.h"
#include "macreadwrite_m.h"
#include <stdio.h>
#include <stdlib.h>

#define NODEBUG


int macseek(MFILE *mfile, SIGNEDLONGWORD offset, int whence) 
{
  lockcatrec(mfile->cr);
  switch (whence) {
  case SEEKFROMBEGINING:
    mfile->offset=(unsigned)offset;
    break;
  case SEEKFROMEND:
    mfile->offset=mfile->cr->U.cdrFilRec.filLgLen-(unsigned)offset;
    break;
  case SEEKFROMCURPOS:
    mfile->offset=(unsigned)((signed)mfile->offset+offset);
    break;
  default:
    unlockcatrec(mfile->cr);
    return BADWHENCE;
  }
  unlockcatrec(mfile->cr);
  return 0;
}



int macread(MFILE *mfile,
	    LONGWORD numbytes,
	    BYTE *buf,
	    LONGWORD *bytesactuallyread)
{
  int rc;

  if (!isreadable(mfile) || mfile->valid!=VALIDVAL)
    return FILEISNOTREADABLE;

  lockcatrec(mfile->cr);

  if (mfile->cr->status) {
    unlockcatrec(mfile->cr);
    return FILEISGONE;
  }

  rc=mread(mfile->mr,
	   mfile->cr,
	   mfile->offset,
	   numbytes,
	   buf,
	   bytesactuallyread);

  unlockcatrec(mfile->cr);
  
  if (rc) 
    return rc;

  mfile->offset+=*bytesactuallyread;

  return 0;

}

   
int macwrite(MFILE *mfile,
	     LONGWORD numbytes,
	     BYTE *buf,
	     LONGWORD *bytesactuallywritten)
{
  int rc;

  if (!iswriteable(mfile) || mfile->valid!=VALIDVAL)
    return FILEISNOTWRITABLE;

  lockcatrec(mfile->cr);

  if (mfile->cr->status) {
    unlockcatrec(mfile->cr);
    return FILEISGONE;
  }

  rc=mwrite(mfile->mr,
	    mfile->cr,
	    mfile->offset,
	    numbytes,
	    buf,
	    bytesactuallywritten);

  unlockcatrec(mfile->cr);

  if (rc)
    return rc;

  mfile->offset+=*bytesactuallywritten;

  return 0;

}




    
