#include <time.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/time.h>
#include <stdlib.h>
#include <sysent.h>
#include "../Includes/macfs_user.h"

struct timeval tv;

void start_timing()
{
    gettimeofday(&tv, NULL);
}

double end_timing()
{
    struct timeval tv2;
    gettimeofday(&tv2, NULL);
    return(tv2.tv_sec - tv.tv_sec
	   + 1.e-6 * (tv2.tv_usec - tv.tv_usec));
}

#define NODEBUG

#define FILESIZE 5000000L
#define FILENAME ":tmp:testfile"

MOUNT_RECORD mr;


int main()
{
  double TIME;
  int rc;
  MFILE mfile;
  char *devname, *partnum;
  int i;
  LONGWORD blocksize;

  devname=getenv("MACDEVICE");
  partnum=getenv("MACPART");



  if (devname==NULL || partnum==NULL) {
    puts("Please set the environment variables MACDEVICE and MACPART before");
    puts("running any mac utility.  MACDEVICE is like /dev/rz4h or /dev/fd0");
    exit(1);
  }

  rc=macmount(devname, atoi(partnum), &mr);

  if (rc) {
    puts("macmount failed");
    exit(2);
  }
  
  initopencatrectable();

  printf("Running create/write-append/delete with varying blocksizes...\n");
#if 1
  for (blocksize=500L; blocksize<=50000L; blocksize*=10L) {
    printf("Writing %ld file with %ld size blocks...\n", FILESIZE, blocksize);
    start_timing();
    if ((rc=maccreatefile(&mr,FILENAME,3,&mfile))) {
      printf("Can't create file rc=%d\n",rc);
      macunmount(&mr);
      exit(-1);
    }
    if ((rc=writefile(&mfile,blocksize))) {
      printf("write file return rc=%d\n", rc);
      macunmount(&mr);
      exit(-1);
    }
    if ((rc=macclose(&mfile))) {
      printf("macclose returned rc=%d\n",rc);
      macunmount(&mr);
      exit(-2);	
    }
    if ((rc=macdeletefile(&mr,FILENAME))) {
      printf("macdeletefile returned %d\n", rc);
      macunmount(&mr);
      exit(-3);
    }
    TIME=end_timing();
    printf("%lu byte file created/written/deleted in %f s or %f bytes/sec\n",
	   FILESIZE, TIME, ((float)FILESIZE)/ TIME);
  }
#endif
  printf("Creating file for over-writes\n");
  if ((rc=maccreatefile(&mr,FILENAME,3,&mfile))) {
    printf("Can't create file rc=%d\n",rc);
    macunmount(&mr);
    exit(-1);
  }
  if ((rc=writefile(&mfile,50000L))) {
    printf("write file return rc=%d\n", rc);
    exit(-1);
  }
  if ((rc=macclose(&mfile))) {
    printf("macclose returned rc=%d\n",rc);
    macunmount(&mr);
    exit(-2);	
  }

  printf("Overwriting file...\n");

  for (blocksize=500L; blocksize<=50000L; blocksize*=10L) {
    printf("Writing %ld file with %ld size blocks...\n", FILESIZE, blocksize);
    start_timing();
    if ((rc=macopen(&mr,FILENAME,3,&mfile))) {
      printf("Can't open file rc=%d\n",rc);
      macunmount(&mr);
      exit(-1);
    }
    if ((rc=writefile(&mfile,blocksize))) {
      printf("write file return rc=%d\n", rc);
      macclose(&mfile);
      macunmount(&mr);
      exit(-1);
    }
    if ((rc=macclose(&mfile))) {
      printf("macclose returned rc=%d\n",rc);
      macunmount(&mr);
      exit(-2);	
    }
    TIME=end_timing();
    printf("%lu byte file created/written/deleted in %f s or %f bytes/sec\n",
	   FILESIZE, TIME, ((float)FILESIZE)/ TIME);
  }

  printf("Reading file...\n");

  for (blocksize=500L; blocksize<=50000L; blocksize*=10L) {
    printf("Reading %ld file with %ld size blocks...\n", FILESIZE, blocksize);
    start_timing();
    if ((rc=macopen(&mr,FILENAME,3,&mfile))) {
      printf("Can't open file rc=%d\n",rc);
      macunmount(&mr);
      exit(-1);
    }
    if ((rc=readfile(&mfile,blocksize))) {
      printf("read file return rc=%d\n", rc);
      macclose(&mfile);
      macunmount(&mr);
      exit(-1);
    }
    if ((rc=macclose(&mfile))) {
      printf("macclose returned rc=%d\n",rc);
      macunmount(&mr);
      exit(-2);	
    }
    TIME=end_timing();
    printf("%lu byte file opened/read/closed in %f s or %f bytes/sec\n",
	   FILESIZE, TIME, ((float)FILESIZE)/ TIME);
  }

  macdeletefile(&mr,FILENAME);
  macunmount(&mr);

  
  
}




int writefile(MFILE *mfile, long blocksize) 
{
  long writenum;	
  char *buf;
  unsigned rc;	
  LONGWORD actual;
  
  if ((buf=(char*) malloc((size_t)blocksize))==NULL)
    return -1;	
  
  
  for (writenum=0;writenum<(FILESIZE/blocksize); writenum++) {
#ifdef DEBUG
    printf("Doing write number %ld - write(%d,buf,%u)\n", writenum, fd, (unsigned) blocksize);
#endif
    if ((rc=macwrite(mfile,(LONGWORD) blocksize,buf,&actual))) {
      printf("Write failed....rc=%d\n",rc);
      perror("write failed");
      free(buf);
      return rc;
    }
    if (actual!=blocksize) {
      printf("wrote %lu bytes but wanted %lu!\n",actual,blocksize);
      free(buf);
      return -5000;
    }
  }
  
  free(buf);
  
  return 0;
}




int readfile(MFILE *mfile, long blocksize) 
{
  long readnum;	
  char *buf;
  unsigned rc;	
  LONGWORD actual;
  
  
  if ((buf=(char*) malloc((size_t)blocksize))==NULL)
    return -1;	
  
  
  for (readnum=0;readnum<(FILESIZE/blocksize); readnum++) {
#ifdef DEBUG
    printf("Doing read number %ld - read(%d,buf,%u)\n", readnum, fd, (unsigned) blocksize);
#endif
    if ((rc=macread(mfile,(LONGWORD)blocksize, buf, &actual))) {
      printf("read failed....rc=%d\n",rc);
      free(buf);
      return rc;
    }
    if (actual!=blocksize) {
      printf("read %lu bytes but wanted %lu!\n",actual,blocksize);
      free(buf);
      return -5001;
    }

  }
  
  free(buf);
  
  return 0;
}



