#include <stdio.h>
#include "Includes/macuserlevel.h"
#include <ctype.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/file.h>
#include <stdlib.h>
#include <sysent.h>

MOUNT_RECORD mr;

#define BLOCKSIZ 1000000

#define NOCORRECTNESS


int main(int argc, char *argv[])
{
  int fail;
  MFILE mfile;
  int fd;
  int b;
  int rc;
  BYTE buf[BLOCKSIZ];
  LONGWORD bytesread, byteswritten;
  char *devname, *partnum;

  devname=getenv("MACDEVICE");
  partnum=getenv("MACPART");

  if (devname==NULL || partnum==NULL) {
    puts("Please set the environment variables MACDEVICE and MACPART before");
    puts("running any mac utility.  MACDEVICE is like /dev/rz4h or /dev/fd0");
    exit(1);
  }

  
  if (argc!=3) {
    puts("usage: maccp unixrelorabspath/file :my:mac:file");
    puts("       maccp :my:mac:file unixrelorabspath/file");
    exit(0);
  }

  if (macmount(devname,atoi(partnum),&mr)) {
    printf("Sorry, couldn't mount %s partition %s\n", devname, partnum );
    exit(1);
  }

  initopencatrectable();

  fail=0;

  b=0;

  printf("Copying from '%s' to '%s'\n", argv[1], argv[2]);

  if (*(argv[2])==':') {
    if ((fd=open(argv[1], O_RDONLY, 0600))<0) {
      printf("Can't Open %s\n", argv[1]);
      fail=1;
    }
    if ((maccreatefile(&mr, argv[2], WRITE, &mfile))) {
      close(fd);
      printf("Can't Create %s\n", argv[2]);
      fail=1;
    }
    if (!fail) { 
      do {
	printf("Working on %d byte block %d\n",BLOCKSIZ,b);
	b++;
	bytesread=read(fd, buf, BLOCKSIZ);
	printf("Trying to write %d bytes\n", bytesread);

	if (rc=macwrite(&mfile, bytesread, buf, &byteswritten)){
	  printf("macwrite failed, rc=%d\n", rc);
	  break;
	}
#ifdef CORRECTNESS

	if ((rc=checkvbmap(&mr))) {
	  printf("checkvbmap returned %d\n", rc);
	  macunmount(&mr);
	  exit(9);
	}
	
	if ((rc=CheckFsBtree(&mr, &(mr.catalog), 1))) {
	  printf("CheckFsBtree(catalog) Returned %d\n", rc);
	  macunmount(&mr);
	  exit(5);
	}
	
	if ((rc=CheckFsBtree(&mr, &(mr.extents), 0))) {
	  printf("CheckFsBtree(extents) Returned %d\n", rc);
	  macunmount(&mr);
	  exit(6);
	}
	
#endif

/*
	printf("vbmap:%d\n", checkvbmap(&mr));
*/
      } while (bytesread==BLOCKSIZ && bytesread==byteswritten);
      puts("COMPLETED..CLOSING");
      macclose(&mfile);
    }
  } else if (*(argv[1])==':') {
    if ((fd=open(argv[2], O_WRONLY | O_CREAT, 0600))<0) {
      printf("Can't Open/create %s\n", argv[2]);
      fail=1;
    }
    if ((macopen(&mr, argv[1], READ, &mfile))) {
      close(fd);
      printf("Can't Open %s\n", argv[1]);
      fail=1;
    }
    if (!fail) {
      do {
	if (macread(&mfile, BLOCKSIZ, buf, &bytesread)) {
	  puts("macread failed");
	  break;
	}
	byteswritten=write(fd, buf, bytesread);

#ifdef CORRECTNESS

	if ((rc=checkvbmap(&mr))) {
	  printf("checkvbmap returned %d\n", rc);
	  macunmount(&mr);
	  exit(9);
	}
	
	if ((rc=CheckFsBtree(&mr, &(mr.catalog), 1))) {
	  printf("CheckFsBtree(catalog) Returned %d\n", rc);
	  macunmount(&mr);
	  exit(5);
	}
	
	if ((rc=CheckFsBtree(&mr, &(mr.extents), 0))) {
	  printf("CheckFsBtree(extents) Returned %d\n", rc);
	  macunmount(&mr);
	  exit(6);
	}
	
#endif



      } while (bytesread==BLOCKSIZ && bytesread==byteswritten);
      macclose(&mfile);
    }
   } else {
     puts("usage: mac unixrelorabspath/file :my:mac:file");
     puts("       mac :my:mac:file unixrelorabspath/file");
   }

#ifdef CORRECTNESS

	if ((rc=checkvbmap(&mr))) {
	  printf("checkvbmap returned %d\n", rc);
	  macunmount(&mr);
	  exit(9);
	}
	
	if ((rc=CheckFsBtree(&mr, &(mr.catalog), 1))) {
	  printf("CheckFsBtree(catalog) Returned %d\n", rc);
	  macunmount(&mr);
	  exit(5);
	}
	
	if ((rc=CheckFsBtree(&mr, &(mr.extents), 0))) {
	  printf("CheckFsBtree(extents) Returned %d\n", rc);
	  macunmount(&mr);
	  exit(6);
	}
	
#endif




  macunmount(&mr);

  exit(0); 

}
