#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/file.h>
#include <stdlib.h>
#include <sysent.h>
#include <errno.h>
#include <sys/stat.h>
#include "../defs.h"
#include "../errors.h"
#include "../Lock/lock.h"
#include "../mountrec.h"
#include "../LibFuncs/macmount.h"
#include "../DevSupport/readwrite.h"
#include "../Volume/volinfo.h"
#include "../Volume/volbmap.h"
#include "../LibFuncs/catalog.h"
#include "../LibFuncs/catalog_local.h"
#include "../btree/btree.h"
#include "../LibFuncs/macreadwrite.h"
#include "../LibFuncs/catrectable.h"
#include "../Times/mac_time.h"




MOUNT_RECORD MR;

#define BLOCKSIZ 1000

void quit()
{
    macunmount(&MR);
    exit(0);
}


int main(int argc, char *argv[])
{
    int fail;
    FILE *f;
    int b;
    int is_file=1;
    struct stat sbuf;
    char *devname, *partnum,*cdfileloc;
    CATALOG_RECORD CR;
    LONGWORD pid;
    BYTE dir[256],name[256];
    devname=getenv("MACDEVICE");
    partnum=getenv("MACPART");
    cdfileloc=getenv("MACCDLOC");
    
    if (devname==NULL || partnum==NULL || cdfileloc==NULL) {
	puts("Please set the environment variables MACDEVICE, MACPART and MACCDLOC before");
	puts("running any mac utility.  MACDEVICE is like /dev/rz4h or /dev/fd0");
	exit(1);
    }
    
    
    if (argc>2) {
	puts("usage: maccd :my:mac:file");
	exit(0);
    }
    
    if (macmount(devname,atoi(partnum),&MR)) {
	printf("Sorry, couldn't mount %s partition %s\n", devname, partnum );
	exit(1);
    }
    
    initopencatrectable();
    
    fail=0;
    
    b=0;

    if(stat(cdfileloc,&sbuf)== -1){
	if(ENOENT == errno){
	    is_file=0;
	} else {
	printf("count not stat %s\n",cdfileloc);
	}
    }

    if(is_file && ((f=fopen(cdfileloc,"r+"))==NULL)){
	    printf("could not open : %s\n",cdfileloc);
	    quit();
    }

    if(argc==2){
	strcpy(dir,argv[1]);
	if (*(argv[1])==':') {
	    if(macresolvepathname(&MR,dir,&CR,&pid,name)){
		printf("%s : no such directory\n", argv[1]);
		quit();
	    }
	} else {
	    if(is_file){
		fgets(dir,256,f);
		strcat(dir,":");
	    } else {
		dir[0]=':';
		dir[1]=NULL;
	    }
	    
	    strcat(dir,argv[1]);
	    if(macresolvepathname(&MR,dir,&CR,&pid,name)){
		printf("%s : no such directory\n", argv[1]);
		quit();
	    }
	}
    } else {
	dir[0]=':';
	dir[1]=NULL;
	if(macresolvepathname(&MR,dir,&CR,&pid,name)){
	    printf("could not change dir to :\n");
	    quit();
	}
    }

    if(CR.cdrType==CDR_DIR){
	if(is_file) fclose(f);    
	if((f=fopen(cdfileloc,"w"))==NULL){
	    printf("could not open : %s\n",cdfileloc);
	    quit();
	}
    } else {
	printf("%s : not a directory\n", argv[1]);
	quit();
    }
    
    fprintf(f,"%s",dir);
    fclose(f);
    macunmount(&MR);
    
    exit(0); 
    
}

