#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/file.h>
#include <stdlib.h>
#include <sysent.h>
#include <errno.h>
#include <sys/stat.h>
#include <strings.h>
#include "../defs.h"
#include "../errors.h"
#include "../Lock/lock.h"
#include "../mountrec.h"
#include "../LibFuncs/macmount.h"
#include "../DevSupport/readwrite.h"
#include "../Volume/volinfo.h"
#include "../Volume/volbmap.h"
#include "../LibFuncs/catalog.h"
#include "../LibFuncs/catalog_local.h"
#include "../btree/btree.h"
#include "../LibFuncs/macreadwrite.h"
#include "../LibFuncs/catrectable.h"
#include "../Times/mac_time.h"




MOUNT_RECORD MR;

#define BLOCKSIZ 1000


void print_mac_time(long t)
{
    char buf[27];
    t -= 0x7C25783F;
    sprintf(buf,"%s",ctime(&t));
    buf[24]=0;
    printf(buf);
}

void quit()
{
    macunmount(&MR);
    exit(0);
}


int resolve_path(char *cdloc,int is_arg,char *arg,CATALOG_RECORD *CR)
{
    struct stat sbuf;
    BYTE dir[256],name[256];
    int is_file=1;
    LONGWORD pid;
    FILE *f;

    
    if(stat(cdloc,&sbuf)== -1){
	if(ENOENT == errno){
	    is_file=0;
	} else {
	    printf("count not stat %s\n",cdloc);
	}
    }

    if(is_file && ((f=fopen(cdloc,"r+"))==NULL)){
	    printf("could not open : %s\n",cdloc);
	    quit();
    }


    if(is_arg){
	strcpy(dir,arg);
	if (*(arg)==':') {
	    if(macresolvepathname(&MR,dir,CR,&pid,name)){
		printf("%s : no such directory\n", arg);
		quit();
	    }
	} else {
	    if(is_file){
		fgets(dir,256,f);
		strcat(dir,":");
	    } else {
		dir[0]=':';
		dir[1]=NULL;
	    }

	    strcat(dir,arg);
	    if(macresolvepathname(&MR,dir,CR,&pid,name)){
		printf("%s : no such directory\n", arg);
		quit();
	    }
	}
    } else {
	if(is_file){
	    fgets(dir,256,f);
	} else {
	    dir[0]=':';
	    dir[1]=NULL;
	}
	if(macresolvepathname(&MR,dir,CR,&pid,name)){
	    printf("%s : no such directory\n", arg);
	    quit();
	}
    }
    if(is_file)fclose(f);
    return 0;
}

int main(int argc, char *argv[])
{
    int fail;
    int b,rc,i;

    char *devname, *partnum,*cdfileloc;
    CATALOG_RECORD CR;


        CATRECANDNAME *crs;
    devname=getenv("MACDEVICE");
    partnum=getenv("MACPART");
    cdfileloc=getenv("MACCDLOC");

    
    if (devname==NULL || partnum==NULL || cdfileloc==NULL) {
	puts("Please set the environment variables MACDEVICE, MACPART and MACCDLOC before");
	puts("running any mac utility.  MACDEVICE is like /dev/rz4h or /dev/fd0");
	exit(1);
    }
    
    
    if (macmount(devname,atoi(partnum),&MR)) {
	printf("Sorry, couldn't mount %s partition %s\n", devname, partnum );
	exit(1);
    }
    
    initopencatrectable();
    
    fail=0;
    
    b=0;
    resolve_path(cdfileloc,(argc==2),argv[1],&CR);
    rc=mdirentries(&MR, CR.U.cdrDirRec.dirDirID, &crs);
    if (rc<0) {
	printf("macdirentries returned error code %d\n", rc);
	return;
    }
        for (i=0;i<rc;i++) {
	switch (crs[i].cr.cdrType) {
	      case CDR_DIR:
		printf("FOLD ");
		break;
	      case CDR_FILE:
		printf("FILE ");
		break;
	      case CDR_DTHREAD:
		printf("DTHD ");
		break;
	      case CDR_FTHREAD:
		printf("FTHD ");
		break;
	      default:
		printf("???? ");
		break;
	}
	printf("%-33s",crs[i].name);
	switch (crs[i].cr.cdrType) {
	      case CDR_DIR:
		printf(" %5d ", crs[i].cr.U.cdrDirRec.dirDirID);
		print_mac_time(crs[i].cr.U.cdrDirRec.dirCrDat);
		printf("   ");
	/*	print_mac_time(crs[i].cr.U.cdrDirRec.dirMdDat); */
		break;
	      case CDR_FILE:
		printf(" %5d ", crs[i].cr.U.cdrFilRec.filLgLen);
		print_mac_time(crs[i].cr.U.cdrFilRec.filCrDat);
		printf("   ");
/*		print_mac_time(crs[i].cr.U.cdrFilRec.filMdDat); */
		break;
	      case CDR_DTHREAD:
	      case CDR_FTHREAD: 
		printf(" NO ID");
		printf(" NO DATE ");
		printf(" NO DATE ");
		break;
	      default:
		printf("???? ");
		printf("???? ");
		printf("???? ");
		break;
	}
	puts("");
    }
    free(crs);
    

    macunmount(&MR);
    
    exit(0); 
    
}

