#include <stdio.h>
#include "../defs.h"
#include "../errors.h"
#include "../mountrec.h"
#include "../LibFuncs/macmount.h"
#include "../DevSupport/readwrite.h"
#include "../Volume/volinfo.h"
#include "../Volume/volbmap.h"
#include "../LibFuncs/catalog.h"
#include "../LibFuncs/catalog_local.h"
#include "../btree/btree.h"
#include "../LibFuncs/macreadwrite.h"
#include <ctype.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <stdlib.h>
#include <sysent.h>

#define ROOTID 2



void print_mac_time(long t)
{
    char buf[27];
    t -= 0x7C25783F;
    sprintf(buf,"%s",ctime(&t));
    buf[24]=0;
    printf(buf);
}

void dir(MOUNT_RECORD *mr,LONGWORD cwd,char *pname)
{
    CATALOG_RECORD cr;
    char name[MAXFNLENGTH];
    int rc,i;
    CATRECANDNAME *crs;
    LONGWORD pid;


    if(*pname==NULL) pname[0]=':';
    if(macresolvepathname(mr,pname,&cr,&pid,name) < 0) return;
    printf("%d\n",pid);
    rc=mdirentries(mr, cr.U.cdrDirRec.dirDirID, &crs);
    if (rc<0) {
	printf("macdirentries returned error code %d\n", rc);
	return;
    }
    printf("type file                                 id  Created Modified\n"
	   "==================================================================\n");
    for (i=0;i<rc;i++) {
	switch (crs[i].cr.cdrType) {
	      case CDR_DIR:
		printf("FOLD ");
		break;
	      case CDR_FILE:
		printf("FILE ");
		break;
	      case CDR_DTHREAD:
		printf("DTHD ");
		break;
	      case CDR_FTHREAD:
		printf("FTHD ");
		break;
	      default:
		printf("???? ");
		break;
	}
	printf("%-33s",crs[i].name);
	switch (crs[i].cr.cdrType) {
	      case CDR_DIR:
		printf(" %5d ", crs[i].cr.U.cdrDirRec.dirDirID);
		print_mac_time(crs[i].cr.U.cdrDirRec.dirCrDat);
		printf("   ");
		print_mac_time(crs[i].cr.U.cdrDirRec.dirMdDat);
		break;
	      case CDR_FILE:
		printf(" %5d ", crs[i].cr.U.cdrFilRec.filFlNum);
		print_mac_time(crs[i].cr.U.cdrFilRec.filCrDat);
		printf("   ");
		print_mac_time(crs[i].cr.U.cdrFilRec.filMdDat);
		break;
	      case CDR_DTHREAD:
	      case CDR_FTHREAD:
		printf(" NO ID");
		printf(" NO DATE ");
		printf(" NO DATE ");
		break;
	      default:
		printf("???? ");
		printf("???? ");
		printf("???? ");
		break;
	}
	puts("");
    }
    free(crs);
}

int move(MOUNT_RECORD *mr)
{
    int rc;
    char n1[MAXFNLENGTH];
    char n2[MAXFNLENGTH];
    printf("from>");
    gets(n1);
    printf("to>");
    gets(n2);
    rc = macmove(mr,n1,n2);
    if (rc<0) {
	printf("macmove returned error code %d\n", rc);
    }
    return rc;

}

int remove_file(MOUNT_RECORD *mr,char *name)
{
    int rc;
    if(name[0]==NULL) return -1;
    rc =  macdeletefile(mr,name);
    if (rc<0) {
	printf("macdeletefile returned error code %d\n", rc);
    }
    return rc;

}

int remove_dir(MOUNT_RECORD *mr,char *name)
{
    int rc;
    if(name[0]==NULL) return -1;
    rc = macdeletefolder(mr,name);
    if (rc<0) {
	printf("macdeletefolder returned error code %d\n", rc);
    }
    return rc;

}

int make_dir(MOUNT_RECORD *mr,char *name)
{
    int rc;
    if(name[0]==NULL) return -1;
    rc = maccreatefolder(mr,name);
    if (rc<0) {
	printf("maccreatefolder returned error code %d\n", rc);
    }
    return rc;

}

int menu(MOUNT_RECORD *mr)
{

    BYTE        *buf;
    BYTE        inp[256];

    CATALOG_RECORD cr;
    LONGWORD cwd=2;
    LONGWORD tmp_d;

    if ((buf=(char *)malloc(BYTESPERSECT*MAXSECTSPEROP))==NULL)
	return -1;
    while (1) {
	printf("%d>",cwd);
	fflush(stdin);fflush(stdout);fflush(stdin);fflush(stdout);
	bzero(inp,256);
	gets(inp);
	if(strncmp(inp,"ls",2) == 0){
	    tmp_d = cwd;
	    dir(mr,tmp_d,inp+3);
	}
	if(strncmp(inp,"mv",2) == 0){
	    move(mr);
	}
	if(strncmp(inp,"rm",2) == 0){
	    remove_file(mr,inp+3);
	}
	if(strncmp(inp,"rmdir",5) == 0){
	    remove_dir(mr,inp+6);
	}
	if(strncmp(inp,"mkdir",5) == 0){
	    make_dir(mr,inp+6);
	}
	
    }
}

int getcharnobuf()
{
    char c;
    
    fflush(stdin);fflush(stdout); fflush(stdout);
    c=(read(0,&c,1) == 1) ? (unsigned char) c : EOF;
    fflush(stdin);fflush(stdout); fflush(stdout);
    return c;
    
}


int textdump(unsigned char buf[], unsigned long length)
{
    int i;
    
    for (i=0;i<length;i++) {
	if (isalnum(buf[i])) 
	    putchar(buf[i]);
	else
	    putchar('.');
    }
    return 0;
}

int showsect(unsigned char buf[])
{
    int i, j;
    
    
    for (i=0; i<BYTESPERSECT; i++) {
	if (!(i%16)) {
	    if (i!=0) {
		printf("  ");
		for (j=i-16;j<i;j++)
		    putchar(isalnum(buf[j])? buf[j]: '.');
	    }
	    printf("\n%5X: %2x", i, buf[i]);
	    continue;
	}
	if (!(i%8)&(i!=0)) {
	    printf(" - %2x", buf[i]);
	} else {
	    printf(" %2x", buf[i]);
	}
    }
    
    printf("  ");
    for(j=i-16;j<i;j++)
	putchar(isalnum(buf[j])? buf[j]: '.');
    
    
    printf("\n\n");
    
    return 0;
}

int main(int argc, char *argv[])
{
    MOUNT_RECORD mr;
    int rc;
    char *devname, *partnum,*cdfileloc;    
    puts("Hi!");
    
    if (argc!=2) {
	puts("usage: mac <device-or-file>");
	exit(0);
    }

    devname=getenv("MACDEVICE");
    partnum=getenv("MACPART");
        
    if (macmount(devname,atoi(partnum),&mr)) {
	printf("Sorry, couldn't mount %s partition %s\n", devname, partnum );
	exit(1);
    }

    initopencatrectable();
    
    menu(&mr);
    
    if ((rc=macunmount(&mr))) {
	printf("macunmount returned with error code %d\n", rc);
    }
    
    exit(0); 
    
}
