
/* explanations of (hopefully) all system types required in this code:

// in vfs/vnode.h: we cast pointers bewteen struct vnode and struct macfs_node
#define vnode inode_header

// in sys/inode.h:
struct inode_header {
	struct inode_header *ih_chain[2]; // must be first
	u_short ih_type;	// inode type (see below)
	u_short	ih_flag;	// inode flags (see below)
	u_short	ih_count;	// reference count
	u_short	ih_shlockc;	// count of shared locks on inode
	u_short	ih_exlockc;	// count of exclusive locks on inode
	u_short ih_mode;	// mode/type of inode
	dev_t ih_rdev;		// block or char device
	struct socket *ih_socket; // unix socket
	struct vfs *ih_mountedfs; // filesystem mounted here
	struct vfs *ih_fs;	// filesystem we are in
	struct vm_info *ih_vm_info; // pointer to VM info structure
};

// in sys/vfs.h: we cast vfs_data to a pointer to macfs_mount
struct vfs {
	struct vfs	*vfs_next;		// next vfs in vfs list
	struct vfsops	*vfs_op;		// operations on vfs
	struct vnode	*vfs_vnodecovered;	// vnode we mounted on
	int		vfs_flag;		// flags
	int		vfs_bsize;		// native block size
	fsid_t		vfs_fsid;		// file system id
	u_short		vfs_exroot;		// exported fs uid 0 mapping
	short		vfs_exflags;		// exported fs flags
	caddr_t		vfs_data;		// private data
};

// in sys/vfs.h: vfs statistics
struct statfs {
	long f_type;			// type of info, zero for now
	long f_bsize;			// fundamental file system block size
	long f_blocks;			// total blocks in file system
	long f_bfree;			// free block in fs
	long f_bavail;			// free blocks avail to non-superuser
	long f_files;			// total file nodes in file system
	long f_ffree;			// free file nodes in fs
	fsid_t f_fsid;			// file system id
	long f_spare[7];		// spare for later
};

// in sys/vfs.h:
typedef struct {
	long val[2];			// file system id type
} fsid_t;

// in sys/vfs.h:
struct fid {
	u_short		fid_len;		// length of data in bytes
	char		fid_data[MAXFIDSZ];	// data (variable length)
};

// in sys/vfs.h: struct ucred used for permissions checking
#define ucred identity

// in sys/user.h:
struct identity {
	u_short	id_ref;			// reference count
	uid_t	id_uid;			// effective user id
	uid_t	id_ruid;		// real user id
	gid_t	id_gid;			// effective group id
	gid_t	id_rgid;		// real group id
	gid_t	id_groups[NGROUPS];	// groups, 0 terminated
	long	id_pag;			// AFS process authentication group
};

// in vfs/vnode.h: A field value of -1 represents a field whose value
//   is unavailable (getattr) or which is not to be changed (setattr).
struct vattr {
	u_short		va_mode;	// files access mode and type
	short		va_uid;		// owner user id
	short		va_gid;		// owner group id
	int32		va_fsid;	// file system id (dev for now)
	int32		va_nodeid;	// node id
	short		va_nlink;	// number of references to file
	u_int32		va_size;	// file size in bytes (quad?)
	int32		va_blocksize;	// blocksize preferred for i/o
	struct timeval	va_atime;	// time of last access
	struct timeval	va_mtime;	// time of last modification
	struct timeval	va_ctime;	// time file ``created
	dev_t		va_rdev;	// device the file represents
	int32		va_blocks;	// kbytes of disk space held by file
};

// in sys/uio.h:
struct uio {
	struct	iovec *uio_iov;
	int	uio_iovcnt; // length of iovec array
	off_t	uio_offset; // offset into the file to start op at
	int	uio_segflg; // source and destination in kernel
	int	uio_resid; // sum of lengths of iovecs
};

// in sys/iovec.h:
struct iovec {
	caddr_t	iov_base; // base pointer
	int	iov_len; // length of iov_base
};

// in sys/buf.h: Bufhd structures used at the head of the hashed buffer
//         queues. We only need three words for these, so this abbreviated
//          definition saves some space.
struct bufhd
{
	long	b_flags;		// see defines below
	struct	buf *b_forw, *b_back;	// fwd/bkwd pointer in chain
};

// in sys/buf.h:
struct buf
{
	long	b_flags;		// too much goes here to describe
	struct	buf *b_forw, *b_back;	// hash chain (2 way street)
	struct	buf *av_forw, *av_back;	// position on free list if not BUSY
#define b_actf	av_forw			// alternate names for driver queue
#define b_actl	av_back			//    head - isn't history wonderful
	long	b_bcount;		// transfer count
	long	b_bufsize;		// size of allocated buffer
#define b_active b_bcount		// driver queue head: drive active
	short	b_error;		// returned after I/O
	dev_t	b_dev;			// major+minor device name
	union {
	    caddr_t b_addr;		// low order core address
	    int	*b_words;		// words for clearing
	    struct fs *b_fs;		// superblocks
	    struct csum *b_cs;		// superblock summary information
	    struct cg *b_cg;		// cylinder group block
	    struct dinode *b_dino;	// ilist
	    daddr_t *b_daddr;		// indirect block
	} b_un;
	daddr_t	b_blkno;		// block # on device
	long	b_resid;		// words not transferred after error
#define b_errcnt b_resid		// while i/o in progress: # retries
	struct  proc *b_proc;		// proc doing physical or swap I/O
	struct	inode *b_ip;		// inode associated with buffer
	int	(*b_iodone)();		// function called by iodone
	int	b_pfcent;		// center page when swapping cluster
#ifdef	mips
	int	b_command;		// SCSI command in progress
#define b_gid b_command;
#endif	mips
#ifdef	exl
// added the missing fields for System V driver (SCSI) --- csy
	daddr_t b_sector;               // physical sector of disk request
	time_t	b_start;		// request start time
#endif	exl
};

// in sys/types.h
#ifdef	KERNEL
typedef	u_long	off_t;
#else	KERNEL
typedef	long	off_t;
#endif	KERNEL

// in sys/types.h:
typdef short dev_t;

// in sys/types.h:
typedef long daddr_t;

// in sys/types.h:
typedef char * caddr_t;

// in sys/types.h:
typedef u_short uid_t;

// in sys/types.h:
typedef u_short gid_t;

// in sys/time.h:
struct timeval {
	long	tv_sec;		// seconds
	long	tv_usec;	// and microseconds
};

// in sys/dir.h
struct	direct {
	u_long	d_ino;			// inode number of entry
	u_short	d_reclen;		// length of this record
	u_short	d_namlen;		// length of string in d_name
	char	d_name[MAXNAMLEN + 1];	// name must be no longer than this
};

*/
